export P=qca-qt5
export V=2.1.3
export B=1
export MAINTAINER=JuergenFischer

savelog() {
	local f=$1
	rm -f $f.9
	for i in $(seq 8 -1 0); do [ -f "$f.$i" ] && mv $f.$i $f.$(( i + 1 )); done
	[ -f $f ] && mv $f $f.0
}

savelog package.log
exec > >(tee package.log) 2>&1
date

set -e

[ -f qca-$V.tar.xz ] || wget http://download.kde.org/stable/qca/$V/src/qca-$V.tar.xz
if ! [ -f ../CMakeLists.txt ]; then
	tar -C .. -xJf qca-$V.tar.xz --xform s,qca-$V,.,
	patch -p1 -d .. <qca.diff
fi

if ! [ -d "$OSGEO4W_REP" ]; then
        echo no repository
        exit 1
fi

fetchenv() {
        local IFS
        IFS=
        batch=$1
        shift

        if ! [ -f "$batch" ]; then
                echo "$batch not found"
                return 1
        fi

        srcenv=$(mktemp /tmp/srcenv.XXXXXXXXXX)
        dstenv=$(mktemp /tmp/dstenv.XXXXXXXXXX)
        diffenv=$(mktemp /tmp/diffenv.XXXXXXXXXX)
        envdiff=$(mktemp /tmp/envdiff.XXXXXXXXXX)
        cmd /c set >$srcenv
        cmd /c call $(cygpath -w $batch) "$@" \>nul 2\>nul \& set >$dstenv
        /usr/bin/cat <<EOF >$envdiff
/^+++/d;
/^[^+]/d;
s/^+//;
# s/\\\\/\\\\\\\\/g;
/^PATH=/ {
        s/\\\\/\\//g;
        s#\\([=;]\\)\\([a-zA-Z]\\):#\\1/cygdrive/\\2#g;
        s/;/:/g;
}
s/;/\\\\;/g;
s/^/export /;
s/=/='/;
s/\\\\r//g;
s/\r$/'/;
EOF
        /usr/bin/diff -u $srcenv $dstenv | /usr/bin/sed -f $envdiff >$diffenv
        . $diffenv
        PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
        rm -f $srcenv $dstenv $diffenv $envdiff
}

for ARCH in x86 x86_64; do
	export ARCH

	(
		case "$ARCH" in
		x86)
        		export OSGEO4W_ROOT=/c/OSGeo4W
			export CMAKE_GEN="Visual Studio 14 2015"
        		export VCARCH=x86
			;;

		x86_64)
			export OSGEO4W_ROOT=/c/OSGeo4W64
	        	export VCARCH=amd64
			export CMAKE_GEN="Visual Studio 14 2015 Win64"
			;;

		*)
        		echo invalid arch
        		exit 1
			;;

		esac

		export R=$OSGEO4W_REP/$ARCH/release/qt5/$P
		mkdir -p $R

		for i in devel libs; do
			mkdir -p $R/$P-$i
			cp ../COPYING $R/$P-$i
		done

		cat <<EOF >$R/$P-devel/setup.hint
sdesc: "Qt5 Cryptographic Architecture headers and libraries (Development)"
ldesc: "Qt5 Cryptographic Architecture headers and libraries (Development)"
maintainer: $MAINTAINER
category: Libs
requires: shell qt5-devel qt5-libs
external-source: qca-qt5-libs
EOF

		cat <<EOF >$R/$P-libs/setup.hint
sdesc: "Qt5 Cryptographic Architecture runtime libraries"
ldesc: "Qt5 Cryptographic Architecture runtime libraries"
maintainer: $MAINTAINER
category: Libs
requires: qt5-libs
EOF

		fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat
		fetchenv $OSGEO4W_ROOT/bin/qt5_env.bat

		fetchenv "/c/Program Files (x86)/Microsoft Visual Studio 14.0/VC/vcvarsall.bat" $VCARCH
		export PATH="$PATH:/c/Program Files/CMake/bin:/bin"

		export INSTDIR=$(cygpath -m /tmp/install-$ARCH)
		rm -rf $INSTDIR
		mkdir -p $INSTDIR

		mkdir -p build-$ARCH
		cd build-$ARCH

		export LIB="$LIB;$(cygpath -m $OSGEO4W_ROOT/lib)"
		export INCLUDE="$INCLUDE;$(cygpath -m $OSGEO4W_ROOT/include)"
		export BUILDCONF=RelWithDebInfo

		cmake -G "$CMAKE_GEN" \
			-Wno-dev \
			-DCMAKE_CONFIGURATION_TYPES=$BUILDCONF \
			-DQCA_PREFIX_INSTALL_DIR=$INSTDIR/apps/Qt5 \
			-DQCA_PLUGINS_INSTALL_DIR=$INSTDIR/apps/Qt5/plugins/ \
			-DQCA_BINARY_INSTALL_DIR=$INSTDIR/apps/Qt5/bin \
			-DQCA_LIBRARY_INSTALL_DIR=$INSTDIR/apps/Qt5/lib \
			-DQCA_FEATURE_INSTALL_DIR=$INSTDIR/apps/Qt5/mkspecs/features/ \
			-DQCA_INCLUDE_INSTALL_DIR=$INSTDIR/apps/Qt5/include \
			-DQCA_PRIVATE_INCLUDE_INSTALL_DIR=$INSTDIR/apps/Qt5/include \
			-DQCA_DOC_INSTALL_DIR=$INSTDIR/apps/Qt5/doc/html/qca \
			-DQCA_MAN_INSTALL_DIR=$INSTDIR/apps/Qt5/man/ \
			-DQCA_SUFFIX=qt5 \
			-DBUILD_TESTS=OFF \
			-DPKGCONFIG_INSTALL_PREFIX=$INSTDIR/apps/Qt5/lib/pkgconfig \
        		../..

		for i in ZERO_CHECK ALL_BUILD INSTALL; do
			echo "$i: $(date)"
			cmake --build . --target $i --config $BUILDCONF
		done

		cd ..

		/usr/bin/tar -C .. -cvjf $R/$P-libs/$P-libs-$V-$B-src.tar.bz2 \
			osgeo4w/package.sh \
			osgeo4w/qca.diff

		cd $INSTDIR

		/usr/bin/tar -cvjf $R/$P-devel/$P-devel-$V-$B.tar.bz2 \
			apps/Qt5/include/QtCrypto \
			apps/Qt5/lib/*.lib \
			apps/Qt5/bin/*.pdb \
			apps/Qt5/mkspecs \
			apps/Qt5/lib/cmake/ \
			apps/Qt5/plugins/crypto/*.pdb

		/usr/bin/tar -cvjf $R/$P-libs/$P-libs-$V-$B.tar.bz2 \
			apps/Qt5/certs \
			apps/Qt5/bin/*.dll \
			apps/Qt5/bin/*.exe \
			apps/Qt5/plugins/crypto/*.dll \
			apps/Qt5/man/man1/qcatool-qt5.1
	)
done

date
